@@ -1,4 +1,4 @@
-/* $Id: VBoxNetAdp-linux.c 135976 2020-02-04 10:35:17Z bird $ */
+/* $Id: VBoxNetAdp-linux.c 139853 2020-08-11 19:53:38Z bird $ */
 /** @file
  * VBoxNetAdp - Virtual Network Adapter Driver (Host), Linux Specific Code.
  */
@@ -75,20 +75,20 @@ static void __exit VBoxNetAdpLinuxUnload(void);
 
 static int VBoxNetAdpLinuxOpen(struct inode *pInode, struct file *pFilp);
 static int VBoxNetAdpLinuxClose(struct inode *pInode, struct file *pFilp);
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 36)
+#if RTLNX_VER_MAX(2,6,36)
 static int VBoxNetAdpLinuxIOCtl(struct inode *pInode, struct file *pFilp,
                                 unsigned int uCmd, unsigned long ulArg);
-#else /* LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36) */
+#else  /* >= 2,6,36 */
 static long VBoxNetAdpLinuxIOCtlUnlocked(struct file *pFilp,
                                          unsigned int uCmd, unsigned long ulArg);
-#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36) */
+#endif /* >= 2,6,36 */
 
 static void vboxNetAdpEthGetDrvinfo(struct net_device *dev, struct ethtool_drvinfo *info);
-# if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0)
+#if RTLNX_VER_MIN(4,20,0)
 static int vboxNetAdpEthGetLinkSettings(struct net_device *pNetDev, struct ethtool_link_ksettings *pLinkSettings);
-#else /* LINUX_VERSION_CODE < KERNEL_VERSION(4, 20, 0) */
+#else  /* < 4,20,0 */
 static int vboxNetAdpEthGetSettings(struct net_device *dev, struct ethtool_cmd *cmd);
-#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(4, 20, 0) */
+#endif /* < 4,20,0 */
 
 
 /*********************************************************************************************************************************
@@ -112,11 +112,11 @@ static struct file_operations gFileOpsVBoxNetAdp =
     owner:      THIS_MODULE,
     open:       VBoxNetAdpLinuxOpen,
     release:    VBoxNetAdpLinuxClose,
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 36)
+#if RTLNX_VER_MAX(2,6,36)
     ioctl:      VBoxNetAdpLinuxIOCtl,
-#else /* LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36) */
+#else /* RTLNX_VER_MIN(2,6,36) */
     unlocked_ioctl: VBoxNetAdpLinuxIOCtlUnlocked,
-#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36) */
+#endif /* RTLNX_VER_MIN(2,6,36) */
 };
 
 /** The miscdevice structure. */
@@ -125,23 +125,23 @@ static struct miscdevice g_CtlDev =
     minor:      MISC_DYNAMIC_MINOR,
     name:       VBOXNETADP_CTL_DEV_NAME,
     fops:       &gFileOpsVBoxNetAdp,
-# if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 17)
+# if RTLNX_VER_MAX(2,6,18)
     devfs_name: VBOXNETADP_CTL_DEV_NAME
 # endif
 };
 
-# if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 19)
+# if RTLNX_VER_MIN(2,6,19)
 static const struct ethtool_ops gEthToolOpsVBoxNetAdp =
 # else
 static struct ethtool_ops gEthToolOpsVBoxNetAdp =
 # endif
 {
     .get_drvinfo        = vboxNetAdpEthGetDrvinfo,
-# if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0)
+# if RTLNX_VER_MIN(4,20,0)
     .get_link_ksettings = vboxNetAdpEthGetLinkSettings,
-# else /* LINUX_VERSION_CODE < KERNEL_VERSION(4, 20, 0) */
+# else
     .get_settings       = vboxNetAdpEthGetSettings,
-# endif /* LINUX_VERSION_CODE < KERNEL_VERSION(4, 20, 0) */
+# endif
     .get_link           = ethtool_op_get_link,
 };
 
@@ -173,7 +173,7 @@ static int vboxNetAdpLinuxXmit(struct sk_buff *pSkb, struct net_device *pNetDev)
     /* Update the stats. */
     pPriv->Stats.tx_packets++;
     pPriv->Stats.tx_bytes += pSkb->len;
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
+#if RTLNX_VER_MAX(2,6,31)
     /* Update transmission time stamp. */
     pNetDev->trans_start = jiffies;
 #endif
@@ -213,7 +213,7 @@ static void vboxNetAdpEthGetDrvinfo(struct net_device *pNetDev, struct ethtool_d
 }
 
 
-# if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0)
+# if RTLNX_VER_MIN(4,20,0)
 /* ethtool_ops::get_link_ksettings */
 static int vboxNetAdpEthGetLinkSettings(struct net_device *pNetDev, struct ethtool_link_ksettings *pLinkSettings)
 {
@@ -229,13 +229,13 @@ static int vboxNetAdpEthGetLinkSettings(struct net_device *pNetDev, struct ethto
     pLinkSettings->base.autoneg     = AUTONEG_DISABLE;
     return 0;
 }
-#else /* LINUX_VERSION_CODE < KERNEL_VERSION(4, 20, 0) */
+#else /* RTLNX_VER_MAX(4,20,0) */
 /* ethtool_ops::get_settings */
 static int vboxNetAdpEthGetSettings(struct net_device *pNetDev, struct ethtool_cmd *cmd)
 {
     cmd->supported      = 0;
     cmd->advertising    = 0;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
+#if RTLNX_VER_MIN(2,6,27)
     ethtool_cmd_speed_set(cmd, SPEED_10);
 #else
     cmd->speed          = SPEED_10;
@@ -249,10 +249,10 @@ static int vboxNetAdpEthGetSettings(struct net_device *pNetDev, struct ethtool_c
     cmd->maxrxpkt       = 0;
     return 0;
 }
-#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(4, 20, 0) */
+#endif /* RTLNX_VER_MAX(4,20,0) */
 
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 29)
+#if RTLNX_VER_MIN(2,6,29)
 static const struct net_device_ops vboxNetAdpNetdevOps = {
     .ndo_open               = vboxNetAdpLinuxOpen,
     .ndo_stop               = vboxNetAdpLinuxStop,
@@ -266,14 +266,14 @@ static void vboxNetAdpNetDevInit(struct net_device *pNetDev)
     PVBOXNETADPPRIV pPriv;
 
     ether_setup(pNetDev);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 29)
+#if RTLNX_VER_MIN(2,6,29)
     pNetDev->netdev_ops = &vboxNetAdpNetdevOps;
-#else /* LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 29) */
+#else /* RTLNX_VER_MAX(2,6,29) */
     pNetDev->open = vboxNetAdpLinuxOpen;
     pNetDev->stop = vboxNetAdpLinuxStop;
     pNetDev->hard_start_xmit = vboxNetAdpLinuxXmit;
     pNetDev->get_stats = vboxNetAdpLinuxGetStats;
-#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 29) */
+#endif /* RTLNX_VER_MAX(2,6,29) */
 
     pNetDev->ethtool_ops = &gEthToolOpsVBoxNetAdp;
 
@@ -290,7 +290,7 @@ int vboxNetAdpOsCreate(PVBOXNETADP pThis, PCRTMAC pMACAddress)
     /* No need for private data. */
     pNetDev = alloc_netdev(sizeof(VBOXNETADPPRIV),
                            pThis->szName[0] ? pThis->szName : VBOXNETADP_LINUX_NAME,
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 17, 0)
+#if RTLNX_VER_MIN(3,17,0)
                            NET_NAME_UNKNOWN,
 #endif
                            vboxNetAdpNetDevInit);
@@ -390,13 +390,13 @@ static int VBoxNetAdpLinuxClose(struct inode *pInode, struct file *pFilp)
  * @param   uCmd        The function specified to ioctl().
  * @param   ulArg       The argument specified to ioctl().
  */
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 36)
+#if RTLNX_VER_MAX(2,6,36)
 static int VBoxNetAdpLinuxIOCtl(struct inode *pInode, struct file *pFilp,
                                 unsigned int uCmd, unsigned long ulArg)
-#else /* LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36) */
+#else /* RTLNX_VER_MIN(2,6,36) */
 static long VBoxNetAdpLinuxIOCtlUnlocked(struct file *pFilp,
                                          unsigned int uCmd, unsigned long ulArg)
-#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36) */
+#endif /* RTLNX_VER_MIN(2,6,36) */
 {
     VBOXNETADPREQ Req;
     PVBOXNETADP pAdp;
